library(plotrix) #package to draw circles easily
library(spatstat) #package for spatial statistics
options(unzip = 'internal') ; install_github('myllym/spptest', ref = 'no_fastdepth') ; library(spptest)  #package to compute extreme rank envelopes
library(TDA) #package to compute  Delaunay-complex filtrations
library(doParallel)  #To parallelize the calculations

# Modification of the TDA function to be compatible with ppp objects from the R-package spatstat
# and to return a matrix in place of object of class diagram
alphaDiag=function(PP){ 
  if (is.ppp(PP)==TRUE ) {
    sim_PP_coord=cbind( PP$x, PP$y ) 
  } else {
    sim_PP_coord=cbind(PP[,1], PP[,2] ) 
  }
  Diag=cbind(alphaComplexDiag(sim_PP_coord)$diagram[,1],sqrt(alphaComplexDiag(sim_PP_coord)$diagram[,2]),sqrt(alphaComplexDiag(sim_PP_coord)$diagram[,3]))
  colnames(Diag)=c("dimension","Birth","Death")
  return(Diag)
}

# Definition of the Accumulated Persistent Function (APF).
# m: the arguments of the APF, Diag: the persistent diagramm, k: the dimension of the topological features
APFtemp=function(m,Diag,k) {
  Diagk=Diag[Diag[,1]==k,]
  if ( is.matrix(Diagk)==TRUE) { lifetime=Diagk[,3]-Diagk[,2] ; meanage= 0.5 * (Diagk[,3]+Diagk[,2] ) ;  sum(lifetime[meanage<=m]) }
  else  { lifetime=Diagk[3]-Diagk[2] ; meanage= 0.5 * (Diagk[3]+Diagk[2] ) ;  sum(lifetime[meanage<=m]) }
} 
APF=Vectorize(APFtemp,vectorize.args = "m","Diag")  



mm=seq(0,0.9,0.0005) 
N=2499 # 2499 number of IID realisation to compute the extreme rank enveloppe
NN=2*2499 # to combine APF0 and APF1, it is recommendanded to use at least 2*2499 
# in "Multiple Monte Carlo testing, with applications in spatial point processes", Myllymaki et al. 2016 
n.sim=500 #number of realisation of each point processes

## Point processes simulation
nbrepoint= 100 #intensity of the point processes, is set to 100 and 400 in the paper.
Ppoisson=rpoispp(lambda = nbrepoint,  win=square(1), nsim=N) # Poisson point process
Pcell= rcell(win=square(1), dx=1/sqrt(nbrepoint), dy=1/sqrt(nbrepoint), N=10, nsim=n.sim, drop=TRUE) # Cell point process
Pmatclust=rMatClust(kappa=10,scale=0.02,mu=nbrepoint/10,win=square(1),nsim=n.sim) #Matérn cluster point process
model = dppBessel(lambda=nbrepoint, alpha=1/sqrt(nbrepoint*pi)-0.01,sigma=0, d=2) # DPP point process simulation
Pdpp=simulate(model,W=square(1),nsim=n.sim)


# Computation of the APFs to compute the enveloppe
registerDoParallel(cores=5)
DiagPoisson= foreach(ii=1:N) %dopar% { alphaDiag(Ppoisson[[ii]]) }
Ppoisson_APF0= foreach(ii=1:N, .combine=cbind) %dopar% { APF( mm, Diag=DiagPoisson[[ii]], k=0 )  }
Ppoisson_APF1= foreach(ii=1:N, .combine=cbind) %dopar% { APF( mm, Diag=DiagPoisson[[ii]], k=1 )  }

 
#Computation of the p-values of the extreme rank enveloppe test
rank.poisson_0=function(XX) { rank_envelope( create_curve_set( 
  list( "r"=mm, "obs"=APF(mm,  alphaDiag(XX)  ,k=0) , "sim_m" = Ppoisson_APF0 ) ) )$p_interval[2]  }
rank.poisson_1=function(XX) { rank_envelope( create_curve_set( 
  list( "r"=mm, "obs"=APF(mm,  alphaDiag(XX)  ,k=1) , "sim_m" = Ppoisson_APF1 ) ) )$p_interval[2]  }
rank.env.comb=function(PP) {  stat.list=list()
  stat.list[[1]]=create_curve_set(list( "r"=mm, "obs"=APF(mm,alphaDiag(PP),k=0), "sim_m" = Ppoisson_APF0))
  stat.list[[2]]=create_curve_set(list( "r"=mm, "obs"=APF(mm,alphaDiag(PP),k=1), "sim_m" = Ppoisson_APF1))
  rank_envelope(combine_curve_sets(stat.list))$p_interval[2]  }

# Realisations of the tests based on APF0 and APF1
H0.test.poisson=foreach(ii=1:n.sim, .combine=c) %dopar% { rank.poisson_0(Ppoisson[[ii]]) }
H0.test.cell=foreach(ii=1:n.sim, .combine=c) %dopar% { rank.poisson_0(Pcell[[ii]]) }
H0.test.clust=foreach(ii=1:n.sim, .combine=c) %dopar% { rank.poisson_0(Pmatclust[[ii]]) }
H0.test.dpp=foreach(ii=1:n.sim, .combine=c) %dopar% { rank.poisson_0(Pdpp[[ii]]) } 
H1.test.poisson=foreach(ii=1:n.sim, .combine=c) %dopar% { rank.poisson_1(Ppoisson[[ii]]) }
H1.test.cell=foreach(ii=1:n.sim, .combine=c) %dopar% { rank.poisson_1(Pcell[[ii]]) }
H1.test.clust=foreach(ii=1:n.sim, .combine=c) %dopar% { rank.poisson_1(Pmatclust[[ii]]) }
H1.test.dpp=foreach(ii=1:n.sim, .combine=c) %dopar% { rank.poisson_1(Pdpp[[ii]]) }
Hcomb.test.poisson=foreach(ii=1:n.sim, .combine=c) %dopar% { rank.env.comb(Ppoisson[[ii]]) }
Hcomb.test.cell=foreach(ii=1:n.sim, .combine=c) %dopar% { rank.env.comb(Pcell[[ii]]) }
Hcomb.test.clust=foreach(ii=1:n.sim, .combine=c) %dopar% { rank.env.comb(Pmatclust[[ii]]) }
Hcomb.test.dpp=foreach(ii=1:n.sim, .combine=c) %dopar% { rank.env.comb(Pdpp[[ii]]) }

## Display the percentage of rejection
error.test.H0 = c(mean(H0.test.poisson<0.05), mean(H0.test.cell<0.05) , mean(H0.test.clust<0.05) , mean(H0.test.dpp<0.05) )
error.test.H0 # When connected components are considered
error.test.H1 = c(mean(H1.test.poisson<0.05), mean(H1.test.cell<0.05) , mean(H1.test.clust<0.05) , mean(H1.test.dpp<0.05) )
error.test.H1 # When loops are considered
error.test.Hcomb = c(mean(Hcomb.test.poisson<0.05), mean(Hcomb.test.cell<0.05) , mean(Hcomb.test.clust<0.05) , mean(Hcomb.test.dpp<0.05) )
error.test.Hcomb # When bot connected components and loops are considered


# Bounds of the enveloppes
lower.env.curve.poisson.0=rank_envelope( create_curve_set( list( "r"=mm, "obs"=APF(mm,  alphaDiag(Pcell[[1]])  ,k=0) , "sim_m" = Ppoisson_APF0 ) ) )$lower
upper.env.curve.poisson.0=rank_envelope( create_curve_set( list( "r"=mm, "obs"=APF(mm,  alphaDiag(Pcell[[1]])  ,k=0) , "sim_m" = Ppoisson_APF0 ) ) )$upper
lower.env.curve.poisson.1=rank_envelope( create_curve_set( list( "r"=mm, "obs"=APF(mm,  alphaDiag(Pcell[[1]])  ,k=1) , "sim_m" = Ppoisson_APF1 ) ) )$lower
upper.env.curve.poisson.1=rank_envelope( create_curve_set( list( "r"=mm, "obs"=APF(mm,  alphaDiag(Pcell[[1]])  ,k=1) , "sim_m" = Ppoisson_APF1 ) ) )$upper


# Code for Figure 4 Left panel
par(mar=c(7,7,0.5,0.5))
plot( mm, upper.env.curve.poisson.0,type="l" ,main="", xlab="" ,ylab="",lwd=0,cex.axis=2.5 ,xlim=c(0,0.45),mgp=c(3,2,0) )
title(ylab=expression(APF[0](m)), mgp=c(4,1,0) ,cex.lab=3)
title(xlab=expression("m") , mgp=c(5,1,0) ,cex.lab=3)
lines(mm, lower.env.curve.poisson.0,lwd=4)
lines(mm, upper.env.curve.poisson.0,lwd=4)
xx=c(mm,rev(mm)) ; yy=c(lower.env.curve.poisson.0,rev(upper.env.curve.poisson.0))
polygon(xx,yy,col=16) 
which(H0.test.clust<0.05)  #to choose a curve corresponding to a case of rejection
lines(mm,APF(mm,  alphaDiag(Pmatclust[[1]])  ,k=0),col=2 ,lwd=6,lty=2)
which(H0.test.dpp<0.05)
lines(mm,APF(mm,  alphaDiag(Pdpp[[1]])  ,k=0),col=4 ,lwd=6,lty=4)
which(H0.test.cell<0.05)
lines(mm,APF(mm,  alphaDiag(Pcell[[11]])  ,k=0),col=6 ,lwd=6,lty=5)
legend("bottomright",c("Extreme rank envelope bounds","Matern cluster","Determinantal","Baddeley-Silverman") 
       ,col=c("1","2","4","6"), lty=c(1,2,4,5) ,y.intersp = 1.5,lwd=4,cex=1.5,seg.len=3)


#Code for Figure 4 Middle panel
par(mar=c(7,7,0.5,0.5))
plot( mm, upper.env.curve.poisson.0,type="l" ,main="", xlab="" ,ylab="",lwd=0,cex.axis=2.5 ,xlim=c(0,0.035),ylim=c(0,2.5),mgp=c(3,2,0) )
title(ylab=expression(APF[0](m)), mgp=c(4,1,0) ,cex.lab=3)
title(xlab=expression("m") , mgp=c(5,1,0) ,cex.lab=3)
lines(mm, lower.env.curve.poisson,lwd=4)
lines(mm, upper.env.curve.poisson,lwd=4)
xx=c(mm,rev(mm)) ; yy=c(lower.env.curve.poisson.0,rev(upper.env.curve.poisson.0))
polygon(xx,yy,col=16) 
which(H0.test.clust<0.05)
lines(mm,APF(mm,  alphaDiag(Pmatclust[[1]])  ,k=0),col=2 ,lwd=6,lty=2)
which(H0.test.dpp<0.05)
lines(mm,APF(mm,  alphaDiag(Pdpp[[1]])  ,k=0),col=4 ,lwd=6,lty=4)
which(H0.test.cell<0.05)
lines(mm,APF(mm,  alphaDiag(Pcell[[11]])  ,k=0),col=6 ,lwd=6,lty=5)
legend("topleft",c("Extreme rank envelope bounds","Matern cluster","Determinantal","Baddeley-Silverman") 
       ,col=c("1","2","4","6"), lty=c(1,2,4,5) ,y.intersp = 1.5,lwd=4,cex=1.5,seg.len=3)


#Code for Figure 4 Right panel
par(mar=c(7,7,0.5,0.5))
plot( mm, upper.env.curve.poisson.1,type="l" ,main="", xlab="" ,ylab="",lwd=0,cex.axis=2.5 ,xlim=c(0,0.6),ylim=c(0,1.8),mgp=c(3,2,0) )
title(ylab=expression(APF[1](m)), mgp=c(4,1,0) ,cex.lab=3)
title(xlab=expression("m") , mgp=c(5,1,0) ,cex.lab=3)
lines(mm, lower.env.curve.poisson.1,lwd=4)
lines(mm, upper.env.curve.poisson.1,lwd=4)
xx=c(mm,rev(mm)) ; yy=c(lower.env.curve.poisson.1,rev(upper.env.curve.poisson.1))
polygon(xx,yy,col=16) 
which(H1.test.clust<0.05)
lines(mm,APF(mm,  alphaDiag(Pmatclust[[2]])  ,k=1),col=2, lwd=6,lty=2)
which(H1.test.dpp<0.05)
lines(mm,APF(mm,  alphaDiag(Pdpp[[3]])  ,k=1),col=4 ,lwd=6,lty=4)
which(H1.test.cell<0.05)
lines(mm,APF(mm,  alphaDiag(Pcell[[18]])  ,k=1),col=6 ,lwd=6,lty=5)
legend("topleft",c("Extreme rank envelope bounds","Matern cluster","Determinantal","Baddeley-Silverman") 
       ,col=c("1","2","4","6"), lty=c(1,2,4,5) ,y.intersp = 1.5,lwd=4,cex=1.5,seg.len=3)



