library(plotrix) #package to draw circles easily
library(doParallel)
library(TDA) #package to compute  Delaunay-complex filtrations

# Modification of the TDA function to be compatible with ppp objects from the R-package spatstat
# and to return a matrix in place of object of class diagram
alphaDiag=function(PP){ 
  if (is.ppp(PP)==TRUE ) {
    sim_PP_coord=cbind( PP$x, PP$y ) 
  } else {
    sim_PP_coord=cbind(PP[,1], PP[,2] ) 
  }
  Diag=cbind(alphaComplexDiag(sim_PP_coord)$diagram[,1],sqrt(alphaComplexDiag(sim_PP_coord)$diagram[,2]),sqrt(alphaComplexDiag(sim_PP_coord)$diagram[,3]))
  colnames(Diag)=c("dimension","Birth","Death")
  return(Diag)
}

# Definition of the Accumulated Persistent Function (APF).
# m: the arguments of the APF, Diag: the persistent diagramm, k: the dimension of the topological features
APFtemp=function(m,Diag,k) {
  Diagk=Diag[Diag[,1]==k,]
  if ( is.matrix(Diagk)==TRUE) { lifetime=Diagk[,3]-Diagk[,2] ; meanage= 0.5 * (Diagk[,3]+Diagk[,2] ) ;  sum(lifetime[meanage<=m]) }
  else  { lifetime=Diagk[3]-Diagk[2] ; meanage= 0.5 * (Diagk[3]+Diagk[2] ) ;  sum(lifetime[meanage<=m]) }
} 
APF=Vectorize(APFtemp,vectorize.args = "m","Diag")  

# function to plot point on the union of three circles
three.unif.circle=function(nbrepoint1,nbrepoint2,nbrepoint3,r1,r2,r3,c1,c2,c3){
  if ( length(c1)==1 ) {  circle1= c1 + circleUnif(nbrepoint1, r = r1) } else {circle1 = cbind( rep(c1[1],nbrepoint1) ,   rep(c1[2],nbrepoint1)) + circleUnif(nbrepoint1, r = r1) }
  if ( length(c2)==1 ) {  circle2= c2 + circleUnif(nbrepoint2, r = r2) } else {circle2 = cbind( rep(c2[1],nbrepoint2) ,   rep(c2[2],nbrepoint2)) + circleUnif(nbrepoint2, r = r2) }
  if ( length(c3)==1 ) {  circle3= c3 + circleUnif(nbrepoint3, r = r3) } else {circle3 = cbind( rep(c3[1],nbrepoint3) ,   rep(c3[2],nbrepoint3)) + circleUnif(nbrepoint3, r = r3) }
  return(rbind(circle1,circle2,circle3))
}

N=50  # Number of copies of point patterns
mm=seq(0,3,0.002)
Pcircle=NULL  
nbrepoint=100  #number of points for each point patterns
# Simulation of the point patterns
for (i in 1:N) {
  repartition=sample(c(1,2,3),nbrepoint,replace=T)
  Pcircle[[i]] = three.unif.circle(nbrepoint1 = length(repartition[repartition==1]) , nbrepoint2 = length(repartition[repartition==2]),
                                   nbrepoint3 = length(repartition[repartition==3]) ,r1=0.5,r2=0.5,r3=0.5,c1=c(-1,-1),c2=c(0,1),c3=c(1,-1))
  
}  

# Computation of the persistence diagram, APF0 and APF1
registerDoParallel(cores=3)
DiagP= foreach(ii=1:N) %dopar% { alphaDiag(Pcircle[[ii]]) }
P_APF0= foreach(ii=1:N, .combine=rbind) %do% { APF( mm, Diag=DiagP[[ii]], k=0 )  }
P_APF1= foreach(ii=1:N, .combine=rbind) %do% { APF( mm, Diag=DiagP[[ii]], k=1 )  }

# Computation of the bounds of the confidence regions for APF0 and APF1
B=1000
diff0=NULL ; diff1=NULL
for (i in 1:B) {
  Nboot=sample(N,N,replace=T)
  Pboot_APF0=P_APF0[Nboot,]
  Pboot_APF1=P_APF1[Nboot,]
  diff0[i]=max( sqrt(N)*abs(colSums(Pboot_APF0)/N - colSums(P_APF0)/N ) )
  diff1[i]=max( sqrt(N)*abs(colSums(Pboot_APF1)/N - colSums(P_APF1)/N ) )
}


# Code for Figure 12 Left panel
par(mar=c(1.5,1.5,0,0))
plot(NULL,asp=1,xlim=c(-1.6,1.6),ylim=c(-1.6,1.6),ann=F,tcl=0,col.axis=0,axes=F)
repartition=sample(c(1,2,3),nbrepoint,replace=T)
points(three.unif.circle(nbrepoint1 = length(repartition[repartition==1]) , nbrepoint2 = length(repartition[repartition==2]),
                         nbrepoint3 = length(repartition[repartition==3]) ,r1=0.5,r2=0.5,r3=0.5,c1=c(-1,-1),c2=c(0,1),c3=c(1,-1)) ,
       main="",xlab="",ylab="",xlim=c(-2.7,1.8),ylim=c(-2.2,2.2),ann=F,tcl=0,col.axis=0,lwd=4,cex=2.5,col=2)
draw.circle(-1,-1,radius=0.5,lwd=4,lty=1) 
draw.circle(0,1,radius=0.5,lwd=4,lty=1)
draw.circle(1,-1,radius=0.5,lwd=4,lty=1)



# Code for Figure 12 Middle panel
par(mar=c(7,7,0.5,0.5))
lower.conf.curve=pmax(0,  -quantile(diff0, probs = 0.95,names=F)/sqrt(N) + colSums(P_APF0) / N)
upper.conf.curve=quantile(diff0,  probs = 0.95,names=F)/sqrt(N) + colSums(P_APF0) / N 
plot(mm,P_APF0[1,],type="l",xlim=c(0,0.5),ylim=c(0,6),col=0,main="", xlab="" ,ylab="",cex.axis=2.5 ,mgp=c(3,2,0) )
title(ylab=expression(APF[0](m)), mgp=c(4,1,0) ,cex.lab=3)
title(xlab=expression("m") , mgp=c(5,1,0) ,cex.lab=3)
lines(mm, lower.conf.curve , col=1,lwd=6 )
lines(mm, upper.conf.curve , col=1,lwd=6 )
xx=c(mm,rev(mm)) ; yy=c(lower.conf.curve,rev(upper.conf.curve))
polygon(xx,yy,col=16) 
lines(mm, colSums(P_APF0) / N, col=2,lwd=6,lty=2)
legend("bottomright",c(expression(atop("Confidence","region bounds")),"Empirical mean")
       ,col=c("1","2"),lty=c(1,2),y.intersp = 1.5,lwd=4,cex=2,seg.len=3)

# Code for Figure 12 Right panel
plot(mm,P_APF1[1,],type="l",xlim=c(0,1),ylim=c(0,1.2),col=0,main="", xlab="" ,ylab="",cex.axis=2.5 ,mgp=c(3,2,0) )
title(ylab=expression(APF[1](m)), mgp=c(4,1,0) ,cex.lab=3)
title(xlab=expression("m") , mgp=c(5,1,0) ,cex.lab=3)
lower.conf.curve=pmax(0,  -quantile(diff1, probs = 0.95,names=F)/sqrt(N) + colSums(P_APF1) / N)
upper.conf.curve=quantile(diff1,  probs = 0.95,names=F)/sqrt(N) + colSums(P_APF1) / N 
lines(mm, lower.conf.curve , col=1,lwd=6 )
lines(mm, upper.conf.curve , col=1,lwd=6 )
xx=c(mm,rev(mm)) ; yy=c(lower.conf.curve,rev(upper.conf.curve))
polygon(xx,yy,col=16) 
lines(mm, colSums(P_APF1) / N, col=2,lwd=6,lty=2)
legend("bottomright",c(expression(atop("Confidence","region bounds")),"Empirical mean")
       ,col=c("1","2"),lty=c(1,2),y.intersp = 1.5,lwd=4,cex=2,seg.len=3)



